import datetime
from unittest import mock

import pytest

from chrono.agendas.models import Agenda, Booking, Event, Subscription
from chrono.utils.lingo import CheckType
from chrono.utils.timezone import make_aware
from tests.utils import login

pytestmark = pytest.mark.django_db


def test_manager_partial_bookings_add_agenda(app, admin_user, settings):
    app = login(app)
    resp = app.get('/manage/agendas/add/')
    assert 'partial-bookings' not in resp.text

    settings.PARTIAL_BOOKINGS_ENABLED = True

    resp = app.get('/manage/agendas/add/')
    resp.form['label'] = 'Foo bar'
    resp.form['kind'] = 'partial-bookings'
    resp = resp.form.submit().follow()

    agenda = Agenda.objects.get(label='Foo bar')
    assert agenda.kind == 'events'
    assert agenda.partial_bookings is True
    assert agenda.default_view == 'day'

    resp = resp.click('Options')
    assert resp.form['default_view'].options == [
        ('', False, '---------'),
        ('day', True, 'Day view'),
        ('month', False, 'Month view'),
    ]


def test_manager_partial_bookings_add_event(app, admin_user):
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)

    app = login(app)
    resp = app.get('/manage/agendas/%s/settings' % agenda.pk)
    resp = resp.click('New Event')
    assert 'duration' not in resp.form.fields
    assert 'recurrence_week_interval' not in resp.form.fields

    resp.form['start_datetime_0'] = '2023-02-15'
    resp.form['start_datetime_1'] = '08:00'
    resp.form['end_time'] = '18:00'
    resp.form['places'] = 10
    resp = resp.form.submit().follow()

    event = Event.objects.get()
    assert event.end_time == datetime.time(18, 00)
    assert event.duration is None

    resp = app.get('/manage/agendas/%s/events/%s/edit' % (agenda.pk, event.pk))
    assert 'duration' not in resp.form.fields
    assert resp.form['end_time'].value == '18:00'

    resp.form['end_time'] = '17:00'
    resp = resp.form.submit().follow()

    resp = app.get('/manage/agendas/%s/settings' % agenda.pk)
    resp = resp.click('New Event')
    resp.form['start_datetime_0'] = '2023-02-15'
    resp.form['start_datetime_1'] = '10:00'
    resp.form['end_time'] = '12:00'
    resp.form['places'] = 10
    resp = resp.form.submit()
    assert 'There can only be one event per day.' in resp.text


def test_manager_partial_bookings_day_view(app, admin_user, freezer):
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 8, 0))
    event = Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(18, 00), places=10, agenda=agenda
    )
    Booking.objects.create(
        user_external_id='xxx',
        user_first_name='Jane',
        user_last_name='Doe',
        start_time=datetime.time(11, 00),
        end_time=datetime.time(13, 30),
        event=event,
    )
    Booking.objects.create(
        user_external_id='yyy',
        user_first_name='Jon',
        user_last_name='Doe',
        start_time=datetime.time(8, 00),
        end_time=datetime.time(10, 00),
        event=event,
    )
    Booking.objects.create(
        user_external_id='zzz',
        user_first_name='Bruce',
        user_last_name='Doe',
        start_time=datetime.time(12, 00),
        end_time=datetime.time(14, 00),
        event=event,
    )

    app = login(app)
    today = start_datetime.date()
    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, today.year, today.month, today.day))
    assert 'Week' not in resp.text

    # time range from one hour before event start to one hour after end
    assert resp.pyquery('.partial-booking--hour')[0].text == '07\u202fh'
    assert resp.pyquery('.partial-booking--hour')[-1].text == '19\u202fh'
    assert [int(x.text.replace('\u202fh', '')) for x in resp.pyquery('.partial-booking--hour')] == list(
        range(7, 20)
    )

    assert len(resp.pyquery('.partial-booking--registrant')) == 3
    assert resp.pyquery('.registrant--name')[0].text_content() == 'Bruce Doe'
    assert resp.pyquery('.registrant--name')[1].text_content() == 'Jane Doe'
    assert resp.pyquery('.registrant--name')[2].text_content() == 'Jon Doe'

    assert resp.pyquery('.registrant--bar')[0].findall('time')[0].text == '12:00'
    assert resp.pyquery('.registrant--bar')[0].findall('time')[1].text == '14:00'
    assert resp.pyquery('.registrant--bar')[0].attrib['style'] == 'left: 38.46%; width: 15.38%;'
    assert resp.pyquery('.registrant--bar')[1].findall('time')[0].text == '11:00'
    assert resp.pyquery('.registrant--bar')[1].findall('time')[1].text == '13:30'
    assert resp.pyquery('.registrant--bar')[1].attrib['style'] == 'left: 30.77%; width: 19.23%;'
    assert resp.pyquery('.registrant--bar')[2].findall('time')[0].text == '08:00'
    assert resp.pyquery('.registrant--bar')[2].findall('time')[1].text == '10:00'
    assert resp.pyquery('.registrant--bar')[2].attrib['style'] == 'left: 7.69%; width: 15.38%;'

    resp = resp.click('Next day')
    assert 'No opening hours this day.' in resp.text


def test_manager_partial_bookings_day_view_24_hours(app, admin_user, freezer):
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 0, 0))
    Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(23, 59), places=10, agenda=agenda
    )

    app = login(app)
    today = start_datetime.date()
    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, today.year, today.month, today.day))

    # from 00h to 23h
    assert [int(x.text.replace('\u202fh', '')) for x in resp.pyquery('.partial-booking--hour')] == list(
        range(0, 24)
    )


@mock.patch('chrono.manager.forms.get_agenda_check_types')
def test_manager_partial_bookings_check(check_types, app, admin_user):
    check_types.return_value = [
        CheckType(slug='foo-reason', label='Foo reason', kind='absence'),
        CheckType(slug='bar-reason', label='Bar reason', kind='presence'),
        CheckType(slug='baz-reason', label='Baz reason', kind='presence'),
    ]
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 8, 0))
    event = Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(18, 00), places=10, agenda=agenda
    )
    Booking.objects.create(
        user_external_id='xxx',
        user_first_name='Jane',
        user_last_name='Doe',
        start_time=datetime.time(11, 00),
        end_time=datetime.time(13, 30),
        event=event,
    )

    app = login(app)
    today = start_datetime.date()
    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, today.year, today.month, today.day))

    assert len(resp.pyquery('.registrant--bar')) == 1
    assert resp.pyquery('.registrant--bar time')[0].text == '11:00'
    assert resp.pyquery('.registrant--bar time')[1].text == '13:30'
    assert resp.pyquery('.registrant--bar')[0].attrib['style'] == 'left: 30.77%; width: 19.23%;'

    resp = resp.click('Jane Doe')
    assert resp.form['presence_check_type'].options == [
        ('', True, '---------'),
        ('bar-reason', False, 'Bar reason'),
        ('baz-reason', False, 'Baz reason'),
    ]
    assert resp.form['absence_check_type'].options == [
        ('', True, '---------'),
        ('foo-reason', False, 'Foo reason'),
    ]

    resp.form['user_check_start_time'] = '11:00'
    resp.form['user_check_end_time'] = '13:15'
    resp.form['user_was_present'] = 'True'
    resp.form['presence_check_type'] = 'bar-reason'
    resp = resp.form.submit().follow()

    assert len(resp.pyquery('.registrant--bar')) == 2
    assert len(resp.pyquery('.registrant--bar.booking')) == 1
    assert len(resp.pyquery('.registrant--bar.check.present')) == 1
    assert resp.pyquery('.registrant--bar.check time')[0].text == '11:00'
    assert resp.pyquery('.registrant--bar.check time')[1].text == '13:15'
    assert resp.pyquery('.registrant--bar.check')[0].attrib['style'] == 'left: 30.77%; width: 17.31%;'
    assert resp.pyquery('.registrant--bar span').text() == 'Bar reason'

    resp = resp.click('Jane Doe')
    assert resp.form['presence_check_type'].value == 'bar-reason'

    resp.form['user_was_present'] = 'False'
    resp = resp.form.submit().follow()

    assert len(resp.pyquery('.registrant--bar')) == 2
    assert len(resp.pyquery('.registrant--bar.booking')) == 1
    assert len(resp.pyquery('.registrant--bar.check.absent')) == 1
    assert resp.pyquery('.registrant--bar span').text() == ''

    resp = resp.click('Jane Doe')
    resp.form['user_was_present'] = ''
    resp = resp.form.submit().follow()
    assert len(resp.pyquery('.registrant--bar')) == 1
    assert len(resp.pyquery('.registrant--bar.booking')) == 1
    assert resp.pyquery('.registrant--bar span').text() == ''


def test_manager_partial_bookings_check_future_events(app, admin_user, freezer):
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 8, 0))
    event = Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(18, 00), places=10, agenda=agenda
    )
    booking = Booking.objects.create(
        user_external_id='xxx',
        user_first_name='Jane',
        user_last_name='Doe',
        start_time=datetime.time(11, 00),
        end_time=datetime.time(13, 30),
        event=event,
    )

    app = login(app)
    day = start_datetime.date()
    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, day.year, day.month, day.day))

    assert len(resp.pyquery('.registrant--name a')) == 1
    app.get('/manage/agendas/%s/bookings/%s/check' % (agenda.pk, booking.pk), status=200)

    freezer.move_to(start_datetime - datetime.timedelta(days=1))

    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, day.year, day.month, day.day))
    assert len(resp.pyquery('.registrant--name a')) == 0
    app.get('/manage/agendas/%s/bookings/%s/check' % (agenda.pk, booking.pk), status=404)

    agenda.enable_check_for_future_events = True
    agenda.save()

    resp = app.get('/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, day.year, day.month, day.day))
    assert len(resp.pyquery('.registrant--name a')) == 1
    app.get('/manage/agendas/%s/bookings/%s/check' % (agenda.pk, booking.pk), status=200)


@mock.patch('chrono.manager.forms.get_agenda_check_types')
def test_manager_partial_bookings_check_filters(check_types, app, admin_user):
    check_types.return_value = [
        CheckType(slug='foo-reason', label='Foo reason', kind='absence'),
        CheckType(slug='bar-reason', label='Bar reason', kind='presence'),
    ]
    agenda = Agenda.objects.create(
        label='Foo bar',
        kind='events',
        partial_bookings=True,
        booking_check_filters='menu',
    )
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 8, 0))
    event = Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(18, 00), places=10, agenda=agenda
    )
    Booking.objects.create(
        user_external_id='user:1',
        user_first_name='User',
        user_last_name='Not Checked',
        start_time=datetime.time(11, 00),
        end_time=datetime.time(13, 30),
        event=event,
    )
    Booking.objects.create(
        user_external_id='user:2',
        user_first_name='User',
        user_last_name='Present Vegan',
        start_time=datetime.time(8, 00),
        end_time=datetime.time(10, 00),
        user_check_start_time=datetime.time(8, 00),
        user_check_end_time=datetime.time(10, 00),
        event=event,
        extra_data={'menu': 'vegan'},
        user_was_present=True,
    )
    Booking.objects.create(
        user_external_id='user:3',
        user_first_name='User',
        user_last_name='Absent Meat Foo Reason',
        start_time=datetime.time(12, 00),
        end_time=datetime.time(14, 00),
        user_check_start_time=datetime.time(12, 30),
        user_check_end_time=datetime.time(14, 30),
        event=event,
        extra_data={'menu': 'meat'},
        user_was_present=False,
        user_check_type_slug='foo-reason',
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:1',
        user_first_name='Subscription',
        user_last_name='Present Vegan',
        date_start=event.start_datetime,
        date_end=event.start_datetime + datetime.timedelta(days=1),
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:4',
        user_first_name='Subscription',
        user_last_name='Not Booked',
        date_start=event.start_datetime,
        date_end=event.start_datetime + datetime.timedelta(days=1),
    )

    app = login(app)
    today = start_datetime.date()
    url = '/manage/agendas/%s/day/%d/%d/%d/' % (agenda.pk, today.year, today.month, today.day)
    resp = app.get(url)
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == [
        'User Absent Meat Foo Reason',
        'Subscription Not Booked',
        'User Not Checked',
        'User Present Vegan',
    ]

    # one registrant has not booked, no bar is shown and no booking check link
    assert len(resp.pyquery('.registrant--bar.booking')) == 3
    assert len(resp.pyquery('.registrant--name a')) == 3

    resp = app.get(url, params={'booking-status': 'booked'})
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == [
        'User Absent Meat Foo Reason',
        'User Not Checked',
        'User Present Vegan',
    ]

    resp = app.get(url, params={'booking-status': 'presence'})
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == ['User Present Vegan']

    resp = app.get(url, params={'booking-status': 'presence', 'extra-data-menu': 'meat'})
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == []

    resp = app.get(url, params={'extra-data-menu': 'meat'})
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == ['User Absent Meat Foo Reason']

    resp = app.get(url, params={'booking-status': 'absence::foo-reason'})
    assert [x.text_content() for x in resp.pyquery('.registrant--name')] == ['User Absent Meat Foo Reason']


def test_manager_partial_bookings_month_view(app, admin_user, freezer):
    agenda = Agenda.objects.create(label='Foo bar', kind='events', partial_bookings=True)
    start_datetime = make_aware(datetime.datetime(2023, 5, 2, 8, 0))
    event = Event.objects.create(
        label='Event', start_datetime=start_datetime, end_time=datetime.time(18, 00), places=10, agenda=agenda
    )
    event2 = Event.objects.create(
        label='Event 2',
        start_datetime=start_datetime + datetime.timedelta(days=2),
        end_time=datetime.time(18, 00),
        places=10,
        agenda=agenda,
    )
    for e in (event, event2):
        Booking.objects.create(
            user_external_id='user:1',
            user_first_name='User',
            user_last_name='Not Checked',
            start_time=datetime.time(11, 00),
            end_time=datetime.time(13, 30),
            event=e,
        )
    Booking.objects.create(
        user_external_id='user:2',
        user_first_name='User',
        user_last_name='Present',
        start_time=datetime.time(8, 00),
        end_time=datetime.time(10, 00),
        user_check_start_time=datetime.time(8, 00),
        user_check_end_time=datetime.time(10, 00),
        event=event,
        user_was_present=True,
    )
    Booking.objects.create(
        user_external_id='user:3',
        user_first_name='User',
        user_last_name='Absent',
        start_time=datetime.time(12, 00),
        end_time=datetime.time(14, 00),
        user_check_start_time=datetime.time(12, 30),
        user_check_end_time=datetime.time(14, 30),
        event=event,
        user_was_present=False,
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:1',
        user_first_name='Subscription',
        user_last_name='Present',
        date_start=event.start_datetime,
        date_end=event.start_datetime + datetime.timedelta(days=1),
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:4',
        user_first_name='Subscription',
        user_last_name='Not Booked',
        date_start=event.start_datetime,
        date_end=event.start_datetime + datetime.timedelta(days=1),
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:5',
        user_first_name='Subscription',
        user_last_name='Next Month',
        date_start=datetime.date(2023, 6, 1),
        date_end=datetime.date(2023, 6, 10),
    )
    Subscription.objects.create(
        agenda=agenda,
        user_external_id='user:6',
        user_first_name='Subscription',
        user_last_name='Previous Month',
        date_start=datetime.date(2023, 4, 20),
        date_end=datetime.date(2023, 4, 30),
    )

    app = login(app)
    today = start_datetime.date()
    resp = app.get('/manage/agendas/%s/month/%d/%d/%d/' % (agenda.pk, today.year, today.month, today.day))

    assert [int(x.text) for x in resp.pyquery('thead th a')] == list(range(1, 32))

    assert [x.text for x in resp.pyquery('tbody tr th')] == [
        'User Absent',
        'Subscription Not Booked',
        'User Not Checked',
        'User Present',
    ]

    user_absent_row = resp.pyquery('tbody tr')[0]
    assert len(resp.pyquery(user_absent_row)('td')) == 31
    assert len(resp.pyquery(user_absent_row)('td span')) == 1
    assert len(resp.pyquery(user_absent_row)('td span.booking.absent')) == 1

    subscription_not_booked_row = resp.pyquery('tbody tr')[1]
    assert len(resp.pyquery(subscription_not_booked_row)('td')) == 31
    assert len(resp.pyquery(subscription_not_booked_row)('td span')) == 0

    user_not_checked_row = resp.pyquery('tbody tr')[2]
    assert len(resp.pyquery(user_not_checked_row)('td')) == 31
    assert len(resp.pyquery(user_not_checked_row)('td span.booking')) == 2

    user_present_row = resp.pyquery('tbody tr')[3]
    assert len(resp.pyquery(user_present_row)('td')) == 31
    assert len(resp.pyquery(user_present_row)('td span')) == 1
    assert len(resp.pyquery(user_present_row)('td span.booking.present')) == 1

    resp = resp.click('Next month')
    assert [int(x.text) for x in resp.pyquery('thead th a')] == list(range(1, 31))
    assert [x.text for x in resp.pyquery('tbody tr th')] == ['Subscription Next Month']
    assert len(resp.pyquery('tbody tr td')) == 30
